@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: route.h 260 2002-02-04 04:03:45Z dugsong $
+ * $Id$
  */
 
 #ifndef DNET_ROUTE_H
@@ -28,7 +28,10 @@ typedef int (*route_handler)(const struct route_entry *entry, void *arg);
 __BEGIN_DECLS
 route_t	*route_open(void);
 int	 route_add(route_t *r, const struct route_entry *entry);
+int	 route_add_dev(route_t *r, const struct route_entry *entry, const char* dev);
+int	 route6_add(route_t *r, const struct route_entry *entry, int intf_index);
 int	 route_delete(route_t *r, const struct route_entry *entry);
+int	 route6_delete(route_t *r, const struct route_entry *entry, int intf_index);
 int	 route_get(route_t *r, struct route_entry *entry);
 int	 route_loop(route_t *r, route_handler callback, void *arg);
 route_t	*route_close(route_t *r);
