@@ -1,4 +1,3 @@
-
 /***************************************************************************
  * netutil.h -- The main include file exposing the external API for        *
  * libnetutil, a library that provides network-related functions or        *
@@ -7,60 +6,59 @@
  * written for use in the Nmap Security Scanner ( https://nmap.org ).       *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- *                                                                         *
- * This program is distributed under the terms of the Nmap Public Source   *
- * License (NPSL). The exact license text applying to a particular Nmap    *
- * release or source code control revision is contained in the LICENSE     *
- * file distributed with that version of Nmap or source code control       *
- * revision. More Nmap copyright/legal information is available from       *
- * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
- * header summarizes some key points from the Nmap license, but is no      *
- * substitute for the actual license text.                                 *
- *                                                                         *
- * Nmap is generally free for end users to download and use themselves,    *
- * including commercial use. It is available from https://nmap.org.        *
- *                                                                         *
- * The Nmap license generally prohibits companies from using and           *
- * redistributing Nmap in commercial products, but we sell a special Nmap  *
- * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem/                                 *
- *                                                                         *
- * If you have received a written Nmap license agreement or contract       *
- * stating terms other than these (such as an Nmap OEM license), you may   *
- * choose to use and redistribute Nmap under those terms instead.          *
- *                                                                         *
- * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.com) for packet capture and transmission. It is under    *
- * separate license terms which forbid redistribution without special      *
- * permission. So the official Nmap Windows builds may not be              *
- * redistributed without special permission (such as an Nmap OEM           *
- * license).                                                               *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to submit your         *
- * changes as a Github PR or by email to the dev@nmap.org mailing list     *
- * for possible incorporation into the main distribution. Unless you       *
- * specify otherwise, it is understood that you are offering us very       *
- * broad rights to use your submissions as described in the Nmap Public    *
- * Source License Contributor Agreement. This is important because we      *
- * fund the project by selling licenses with various terms, and also       *
- * because the inability to relicense code has caused devastating          *
- * problems for other Free Software projects (such as KDE and NASM).       *
- *                                                                         *
- * The free version of Nmap is distributed in the hope that it will be     *
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
- * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem/                            *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
 /* $Id: netutil.h 18098 2010-06-14 11:50:12Z luis $ */
@@ -76,8 +74,8 @@ extern "C" {
 }
 #endif
 
-#include "dnet.h"
 #include <nbase.h>
+#include <dnet.h>
 
 /* It is VERY important to never change the value of these two constants.
  * Specially, OP_FAILURE should never be positive, as some pieces of code take
@@ -167,7 +165,8 @@ int resolve_numeric(const char *ip, unsigned short port,
  * <http://www.cymru.com/Documents/bogon-bn-nonagg.txt> for bogon
  * netblocks.
  */
-int ip_is_reserved(struct in_addr *ip);
+int ip_is_reserved(const struct sockaddr_storage *addr);
+
 
 
 /* A couple of trivial functions that maintain a cache of IP to MAC
@@ -238,7 +237,7 @@ typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;
 struct link_header {
   int datalinktype; /* pcap_datalink(), such as DLT_EN10MB */
   int headerlen; /* 0 if header was too big or unavailaable */
-  u8 header[MAX_LINK_HEADERSZ];
+  const u8 *header;
 };
 
 /* Relevant (to Nmap) information about an interface */
@@ -279,10 +278,12 @@ struct sys_route {
   int metric;
 };
 
+struct netutil_eth_t;
+
 struct eth_nfo {
   char srcmac[6];
   char dstmac[6];
-  eth_t *ethsd; // Optional, but improves performance.  Set to NULL if unavail
+  netutil_eth_t *ethsd; // Optional, but improves performance.  Set to NULL if unavail
   char devname[16]; // Only needed if ethsd is NULL.
 };
 
@@ -294,11 +295,44 @@ struct eth_nfo {
    eth_close() A DEVICE OBTAINED FROM THIS FUNCTION.  Instead, you can
    call eth_close_cached() to close whichever device (if any) is
    cached.  Returns NULL if it fails to open the device. */
-eth_t *eth_open_cached(const char *device);
+netutil_eth_t *eth_open_cached(const char *device);
+netutil_eth_t *netutil_eth_open(const char *device);
+void netutil_eth_close(netutil_eth_t *e);
+ssize_t netutil_eth_send(netutil_eth_t *e, const void *buf, size_t len);
+int netutil_eth_datalink(const netutil_eth_t *e);
+int netutil_eth_can_send(const netutil_eth_t *e);
 
 /* See the description for eth_open_cached */
 void eth_close_cached();
 
+/* Create a raw socket and do things that always apply to raw sockets:
+    * Set SO_BROADCAST.
+    * Set IP_HDRINCL.
+    * Bind to an interface with SO_BINDTODEVICE (if device is not NULL).
+   The socket is created with address family AF_INET, but may be usable for
+   AF_INET6, depending on the operating system. */
+int netutil_raw_socket(const char *device);
+
+/* How should we send raw IP packets?  Nmap can generally use either
+   ethernet or raw ip sockets.  Which is better depends on platform
+   and goals.  A _STRONG preference means that Nmap should use the
+   preferred method whenever it is possible (obviously it isn't
+   always possible -- sending ethernet frames won't work over a PPP
+   connection).  This is useful when the other type doesn't work at
+   all.  A _WEAK preference means that Nmap may use the other type
+   where it is substantially more efficient to do so. For example,
+   Nmap will still do an ARP ping scan of a local network even when
+   the pref is SEND_IP_WEAK */
+#define PACKET_SEND_NOPREF      0x01
+#define PACKET_SEND_ETH_WEAK    0x02
+#define PACKET_SEND_ETH_STRONG  0x04
+#define PACKET_SEND_ETH (PACKET_SEND_ETH_WEAK | PACKET_SEND_ETH_STRONG)
+#define PACKET_SEND_IP_WEAK     0x08
+#define PACKET_SEND_IP_STRONG   0x10
+#define PACKET_SEND_IP (PACKET_SEND_IP_WEAK | PACKET_SEND_IP_STRONG)
+int raw_socket_or_eth(int sendpref, const char *ifname,
+    int *rawsd, netutil_eth_t **ethsd);
+
 /* Takes a protocol number like IPPROTO_TCP, IPPROTO_UDP, or
  * IPPROTO_IP and returns a ascii representation (or "unknown" if it
  * doesn't recognize the number).  Returned string is in lowercase. */
@@ -320,7 +354,7 @@ int ipaddr2devname( char *dev, const struct sockaddr_storage *addr );
 
 /* Convert a network interface name (IE ppp0 eth0) to an IP address.
  * Returns 0 on success or -1 in case of error. */
-int devname2ipaddr(char *dev, struct sockaddr_storage *addr);
+int devname2ipaddr(char *dev, int af, struct sockaddr_storage *addr);
 
 int sockaddr_equal(const struct sockaddr_storage *a,
   const struct sockaddr_storage *b);
@@ -373,12 +407,6 @@ struct sys_route *getsysroutes(int *howmany, char *errstr, size_t errstrlen);
  * Returns 1 if the address is thought to be localhost and 0 otherwise */
 int islocalhost(const struct sockaddr_storage *ss);
 
-/* Determines whether the supplied address corresponds to a private,
- * non-Internet-routable address. See RFC1918 for details.
- * Also checks for link-local addresses per RFC3927.
- * Returns 1 if the address is private or 0 otherwise. */
-int isipprivate(const struct sockaddr_storage *addr);
-
 /* Takes binary data found in the IP Options field of an IPv4 packet
  * and returns a string containing an ASCII description of the options
  * found. The function returns a pointer to a static buffer that
@@ -431,9 +459,6 @@ int route_dst(const struct sockaddr_storage *dst, struct route_nfo *rnfo,
 /* Send an IP packet over a raw socket. */
 int send_ip_packet_sd(int sd, const struct sockaddr_in *dst, const u8 *packet, unsigned int packetlen);
 
-/* Send an IP packet over an ethernet handle. */
-int send_ip_packet_eth(const struct eth_nfo *eth, const u8 *packet, unsigned int packetlen);
-
 /* Sends the supplied pre-built IPv4 packet. The packet is sent through
  * the raw socket "sd" if "eth" is NULL. Otherwise, it gets sent at raw
  * ethernet level. */
@@ -535,9 +560,8 @@ int read_reply_pcap(pcap_t *pd, long to_usec,
 size_t read_host_from_file(FILE *fp, char *buf, size_t n);
 
 /* Return next target host specification from the supplied stream.
- * if parameter "random" is set to true, then the function will
- * return a random, non-reserved, IP address in decimal-dot notation */
-const char *grab_next_host_spec(FILE *inputfd, bool random, int argc, const char **fakeargv);
+ */
+const char *grab_next_host_spec(FILE *inputfd, int argc, const char **fakeargv);
 
 #ifdef WIN32
 /* Convert a dnet interface name into the long pcap style.  This also caches the
