@@ -22,6 +22,23 @@
 
 #include "ncat_core.h"
 
+#ifdef WIN32
+#include "../nsock/src/error.h"
+#endif
+
+
+#ifdef WIN32
+static void win_init(void)
+{
+  WSADATA data;
+  int rc;
+
+  rc = WSAStartup(MAKEWORD(2,2), &data);
+  if (rc)
+    fatal("failed to start winsock: %s\n", socket_strerror(rc));
+}
+#endif
+
 static int resolve_name(const char *name, struct addrinfo **result)
 {
     struct addrinfo hints = { 0 };
@@ -34,16 +51,20 @@ static int resolve_name(const char *name, struct addrinfo **result)
 
 int main(int argc, char *argv[])
 {
-    struct addrset set;
+    struct addrset *set;
     char line[1024];
     int i;
 
-    addrset_init(&set);
+#ifdef WIN32
+    win_init();
+#endif
+
+    set = addrset_new();
 
     options_init();
 
     for (i = 1; i < argc; i++) {
-        if (!addrset_add_spec(&set, argv[i], o.af, !o.nodns)) {
+        if (!addrset_add_spec(set, argv[i], o.af, !o.nodns)) {
             fprintf(stderr, "Error adding spec \"%s\".\n", argv[i]);
             exit(1);
         }
@@ -70,14 +91,14 @@ int main(int argc, char *argv[])
             }
 
             /* Check just the first address returned. */
-            if (addrset_contains(&set, addrs->ai_addr))
+            if (addrset_contains(set, addrs->ai_addr))
                     printf("%s\n", hostname);
 
             freeaddrinfo(addrs);
         }
     }
 
-    addrset_free(&set);
+    addrset_free(set);
 
     return 0;
 }
