@@ -3,7 +3,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *
- * The nsock parallel socket event library is (C) 1999-2024 Nmap Software LLC
+ * The nsock parallel socket event library is (C) 1999-2025 Nmap Software LLC
  * This library is free software; you may redistribute and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation; Version 2. This guarantees your right to use, modify, and
@@ -48,7 +48,7 @@
  *
  ***************************************************************************/
 
-/* $Id: engine_epoll.c 38790 2024-02-28 18:46:45Z dmiller $ */
+/* $Id: engine_epoll.c 39083 2025-02-26 17:44:43Z dmiller $ */
 
 #ifdef HAVE_CONFIG_H
 #include "nsock_config.h"
@@ -104,22 +104,6 @@ struct io_engine engine_epoll = {
 /* --- INTERNAL PROTOTYPES --- */
 static void iterate_through_event_lists(struct npool *nsp, int evcount);
 
-/* defined in nsock_core.c */
-void process_iod_events(struct npool *nsp, struct niod *nsi, int ev);
-void process_event(struct npool *nsp, gh_list_t *evlist, struct nevent *nse, int ev);
-void process_expired_events(struct npool *nsp);
-#if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-int pcap_read_on_nonselect(struct npool *nsp);
-#endif
-#endif
-
-/* defined in nsock_event.c */
-void update_first_events(struct nevent *nse);
-
-
-extern struct timeval nsock_tod;
-
 
 /*
  * Engine specific data structure
@@ -131,6 +115,8 @@ struct epoll_engine_info {
   int evlen;
   /* list of epoll events, resized if necessary (when polling over large numbers of IODs) */
   struct epoll_event *events;
+  /* Number of IODs incompatible with epoll */
+  int num_pcap_nonselect;
 };
 
 
@@ -142,6 +128,7 @@ int epoll_init(struct npool *nsp) {
   einfo->epfd = epoll_create(10); /* argument is ignored */
   einfo->evlen = INITIAL_EV_COUNT;
   einfo->events = (struct epoll_event *)safe_malloc(einfo->evlen * sizeof(struct epoll_event));
+  einfo->num_pcap_nonselect = 0;
 
   nsp->engine_data = (void *)einfo;
 
@@ -166,18 +153,26 @@ int epoll_iod_register(struct npool *nsp, struct niod *iod, struct nevent *nse,
 
   iod->watched_events = ev;
 
-  memset(&epev, 0x00, sizeof(struct epoll_event));
-  epev.events = EPOLLET;
-  epev.data.ptr = (void *)iod;
-
-  if (ev & EV_READ)
-    epev.events |= EPOLL_R_FLAGS;
-  if (ev & EV_WRITE)
-    epev.events |= EPOLL_W_FLAGS;
-
   sd = nsock_iod_get_sd(iod);
-  if (epoll_ctl(einfo->epfd, EPOLL_CTL_ADD, sd, &epev) < 0)
-    fatal("Unable to register IOD #%lu: %s", iod->id, strerror(errno));
+  if (sd == -1) {
+    if (iod->pcap)
+      einfo->num_pcap_nonselect++;
+    else
+      fatal("Unable to get descriptor for IOD #%lu", iod->id);
+  }
+  else {
+    memset(&epev, 0x00, sizeof(struct epoll_event));
+    epev.events = EPOLLET;
+    epev.data.ptr = (void *)iod;
+
+    if (ev & EV_READ)
+      epev.events |= EPOLL_R_FLAGS;
+    if (ev & EV_WRITE)
+      epev.events |= EPOLL_W_FLAGS;
+
+    if (epoll_ctl(einfo->epfd, EPOLL_CTL_ADD, sd, &epev) < 0)
+      fatal("Unable to register IOD #%lu: %s", iod->id, strerror(errno));
+  }
 
   IOD_PROPSET(iod, IOD_REGISTERED);
   return 1;
@@ -193,7 +188,13 @@ int epoll_iod_unregister(struct npool *nsp, struct niod *iod) {
     int sd;
 
     sd = nsock_iod_get_sd(iod);
-    epoll_ctl(einfo->epfd, EPOLL_CTL_DEL, sd, NULL);
+    if (sd == -1) {
+      assert(iod->pcap);
+      einfo->num_pcap_nonselect--;
+    }
+    else {
+      epoll_ctl(einfo->epfd, EPOLL_CTL_DEL, sd, NULL);
+    }
 
     IOD_PROPCLR(iod, IOD_REGISTERED);
   }
@@ -222,16 +223,17 @@ int epoll_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, in
 
   iod->watched_events = new_events;
 
-  /* regenerate the current set of events for this IOD */
-  if (iod->watched_events & EV_READ)
-    epev.events |= EPOLL_R_FLAGS;
-  if (iod->watched_events & EV_WRITE)
-    epev.events |= EPOLL_W_FLAGS;
-
   sd = nsock_iod_get_sd(iod);
+  if (sd != -1) {
+    /* regenerate the current set of events for this IOD */
+    if (iod->watched_events & EV_READ)
+      epev.events |= EPOLL_R_FLAGS;
+    if (iod->watched_events & EV_WRITE)
+      epev.events |= EPOLL_W_FLAGS;
 
-  if (epoll_ctl(einfo->epfd, EPOLL_CTL_MOD, sd, &epev) < 0)
-    fatal("Unable to update events for IOD #%lu: %s", iod->id, strerror(errno));
+    if (epoll_ctl(einfo->epfd, EPOLL_CTL_MOD, sd, &epev) < 0)
+      fatal("Unable to update events for IOD #%lu: %s", iod->id, strerror(errno));
+  }
 
   return 1;
 }
@@ -250,7 +252,7 @@ int epoll_loop(struct npool *nsp, int msec_timeout) {
     return 0; /* No need to wait on 0 events ... */
 
 
-  iod_count = gh_list_count(&nsp->active_iods);
+  iod_count = gh_list_count(&nsp->active_iods) - einfo->num_pcap_nonselect;
   if (iod_count > einfo->evlen) {
     einfo->evlen = iod_count * 2;
     einfo->events = (struct epoll_event *)safe_realloc(einfo->events, einfo->evlen * sizeof(struct epoll_event));
@@ -260,41 +262,49 @@ int epoll_loop(struct npool *nsp, int msec_timeout) {
     struct nevent *nse;
 
     nsock_log_debug_all("wait for events");
+    results_left = 0;
 
     nse = next_expirable_event(nsp);
     if (!nse)
       event_msecs = -1; /* None of the events specified a timeout */
-    else
-      event_msecs = MAX(0, TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod));
+    else {
+      event_msecs = TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod);
+      event_msecs = MAX(0, event_msecs);
+    }
 
 #if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-    /* Force a low timeout when capturing packets on systems where
-     * the pcap descriptor is not select()able. */
-    if (gh_list_count(&nsp->pcap_read_events) > 0)
-      if (event_msecs > PCAP_POLL_INTERVAL)
-        event_msecs = PCAP_POLL_INTERVAL;
-#endif
-#endif
+    if (einfo->num_pcap_nonselect > 0 && gh_list_count(&nsp->pcap_read_events) > 0) {
 
+      /* do non-blocking read on pcap devices that doesn't support select()
+       * If there is anything read, just leave this loop. */
+      if (pcap_read_on_nonselect(nsp)) {
+        /* okay, something was read. */
+        // Check all pcap events that won't be signaled
+        gettimeofday(&nsock_tod, NULL);
+        iterate_through_pcap_events(nsp);
+        // Make the system call non-blocking
+        event_msecs = 0;
+      }
+      /* Force a low timeout when capturing packets on systems where
+       * the pcap descriptor is not select()able. */
+      else if (event_msecs > PCAP_POLL_INTERVAL) {
+        event_msecs = PCAP_POLL_INTERVAL;
+      }
+    }
+#endif
     /* We cast to unsigned because we want -1 to be very high (since it means no
      * timeout) */
     combined_msecs = MIN((unsigned)event_msecs, (unsigned)msec_timeout);
 
-#if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-    /* do non-blocking read on pcap devices that doesn't support select()
-     * If there is anything read, just leave this loop. */
-    if (pcap_read_on_nonselect(nsp)) {
-      /* okay, something was read. */
-    } else
-#endif
-#endif
-    {
+    if (iod_count > 0) {
       results_left = epoll_wait(einfo->epfd, einfo->events, einfo->evlen, combined_msecs);
       if (results_left == -1)
         sock_err = socket_errno();
     }
+    else if (combined_msecs > 0) {
+      // No compatible IODs; sleep the remainder of the wait time.
+      usleep(combined_msecs * 1000);
+    }
 
     gettimeofday(&nsock_tod, NULL); /* Due to epoll delay */
   } while (results_left == -1 && sock_err == EINTR); /* repeat only if signal occurred */
