@@ -4,60 +4,59 @@
  * are related to port scanning using raw (IP, Ethernet) packets.          *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- *                                                                         *
- * This program is distributed under the terms of the Nmap Public Source   *
- * License (NPSL). The exact license text applying to a particular Nmap    *
- * release or source code control revision is contained in the LICENSE     *
- * file distributed with that version of Nmap or source code control       *
- * revision. More Nmap copyright/legal information is available from       *
- * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
- * header summarizes some key points from the Nmap license, but is no      *
- * substitute for the actual license text.                                 *
- *                                                                         *
- * Nmap is generally free for end users to download and use themselves,    *
- * including commercial use. It is available from https://nmap.org.        *
- *                                                                         *
- * The Nmap license generally prohibits companies from using and           *
- * redistributing Nmap in commercial products, but we sell a special Nmap  *
- * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem/                                 *
- *                                                                         *
- * If you have received a written Nmap license agreement or contract       *
- * stating terms other than these (such as an Nmap OEM license), you may   *
- * choose to use and redistribute Nmap under those terms instead.          *
- *                                                                         *
- * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.com) for packet capture and transmission. It is under    *
- * separate license terms which forbid redistribution without special      *
- * permission. So the official Nmap Windows builds may not be              *
- * redistributed without special permission (such as an Nmap OEM           *
- * license).                                                               *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to submit your         *
- * changes as a Github PR or by email to the dev@nmap.org mailing list     *
- * for possible incorporation into the main distribution. Unless you       *
- * specify otherwise, it is understood that you are offering us very       *
- * broad rights to use your submissions as described in the Nmap Public    *
- * Source License Contributor Agreement. This is important because we      *
- * fund the project by selling licenses with various terms, and also       *
- * because the inability to relicense code has caused devastating          *
- * problems for other Free Software projects (such as KDE and NASM).       *
- *                                                                         *
- * The free version of Nmap is distributed in the hope that it will be     *
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
- * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem/                            *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
 /* $Id$ */
@@ -109,6 +108,32 @@ u16 UltraProbe::dport() const {
   /* not reached */
 }
 
+bool UltraProbe::check_proto_port(u8 proto, u16 sport_or_icmpid, u16 dport) const {
+  if (proto != mypspec.proto)
+    return false;
+  switch (proto) {
+    case IPPROTO_ICMP:
+    case IPPROTO_ICMPV6:
+      return sport_or_icmpid == probes.IP.pd.icmp.ident;
+      break;
+    case IPPROTO_TCP:
+      return sport_or_icmpid == probes.IP.pd.tcp.sport &&
+        dport == mypspec.pd.tcp.dport;
+      break;
+    case IPPROTO_UDP:
+      return sport_or_icmpid == probes.IP.pd.udp.sport &&
+        dport == mypspec.pd.udp.dport;
+      break;
+    case IPPROTO_SCTP:
+      return sport_or_icmpid == probes.IP.pd.sctp.sport &&
+        dport == mypspec.pd.sctp.dport;
+      break;
+    default:
+      break;
+  }
+    return false;
+}
+
 /* Pass an arp packet, including ethernet header. Must be 42bytes */
 
 void UltraProbe::setARP(const u8 *arppkt, u32 arplen) {
@@ -269,17 +294,12 @@ static bool sport_decode(u16 base_portno, u16 portno,
 
 static bool icmp_probe_match(const UltraScanInfo *USI, const UltraProbe *probe,
                              const struct ppkt *ping,
-                             const struct sockaddr_storage *target_src,
                              const struct sockaddr_storage *src,
                              const struct sockaddr_storage *dst,
                              u8 proto,
                              u32 ipid) {
   /* Check if it is ICMP or ICMPV6. */
-  if (probe->protocol() != IPPROTO_ICMPV6 && probe->protocol() != IPPROTO_ICMP)
-    return false;
-
-  /* Ensure the connection info matches. */
-  if (sockaddr_storage_cmp(target_src, dst) != 0)
+  if (!probe->check_proto_port(proto, ntohs(ping->id), 0))
     return false;
 
   /* Don't match a timestamp request with an echo reply, for example. */
@@ -300,34 +320,17 @@ static bool icmp_probe_match(const UltraScanInfo *USI, const UltraProbe *probe,
       probe->ipid() == ipid)
     return false; /* We saw the packet we ourselves sent */
 
-  /* Check that the randomly-generated ping ident matches. */
-  if (ntohs(ping->id) != probe->icmpid())
-    return false;
-
   return true;
 }
 
 static bool tcp_probe_match(const UltraScanInfo *USI, const UltraProbe *probe,
-                            const HostScanStats *hss, const struct tcp_hdr *tcp,
+                            const struct tcp_hdr *tcp,
                             const struct sockaddr_storage *src, const struct sockaddr_storage *dst,
                             u32 ipid) {
   const struct probespec_tcpdata *probedata;
-  struct sockaddr_storage srcaddr;
-  size_t srcaddr_len;
   tryno_t tryno = {0};
   bool goodseq;
 
-  if (probe->protocol() != IPPROTO_TCP)
-    return false;
-
-  srcaddr_len = sizeof(srcaddr);
-  hss->target->SourceSockAddr(&srcaddr, &srcaddr_len);
-  /* Ensure the connection info matches. */
-  if (probe->dport() != ntohs(tcp->th_sport)
-      || probe->sport() != ntohs(tcp->th_dport)
-      || sockaddr_storage_cmp(&srcaddr, dst) != 0)
-    return false;
-
   // If magic port is *not* set, then tryno is in the source port, and we
   // already checked that it matches.
   if (o.magic_port_set) {
@@ -401,9 +404,6 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
   unsigned int listsz;
   reason_t current_reason = ER_NORESPONSE;
 
-  struct sockaddr_storage target_src, target_dst;
-  size_t ss_len;
-
   const void *data = NULL;
   unsigned int datalen;
   struct abstract_ip_hdr hdr;
@@ -416,7 +416,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
                                        &linkhdr, true);
     gettimeofday(&USI->now, NULL);
     if (!ip_tmp) {
-      if (TIMEVAL_SUBTRACT(*stime, USI->now) < 0) {
+      if (TIMEVAL_BEFORE(*stime, USI->now)) {
         timedout = true;
         break;
       } else {
@@ -438,6 +438,9 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
     data = ip_get_data(ip_tmp, &datalen, &hdr);
     if (data == NULL)
       continue;
+    // If it's not sent to us, we don't care.
+    if (sockaddr_storage_cmp(USI->SourceSockAddr(), &hdr.dst) != 0)
+      continue;
 
     /* First check if it is ICMP, TCP, or UDP */
     if (hdr.proto == IPPROTO_ICMP || hdr.proto == IPPROTO_ICMPV6) {
@@ -463,9 +466,6 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         probeI = hss->probes_outstanding.end();
         listsz = hss->num_probes_outstanding();
 
-        ss_len = sizeof(target_src);
-        hss->target->SourceSockAddr(&target_src, &ss_len);
-
         /* A check for weird_responses is needed here. This is not currently
            possible because we don't have a good way to look up the original
            target of an ICMP probe based on the response. (massping encoded an
@@ -485,7 +485,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
           probeI--;
           probe = *probeI;
 
-          if (!icmp_probe_match(USI, probe, ping, &target_src, &hdr.src, &hdr.dst, hdr.proto, hdr.ipid))
+          if (!icmp_probe_match(USI, probe, ping, &hdr.src, &hdr.dst, hdr.proto, hdr.ipid))
             continue;
 
           goodone = true;
@@ -535,6 +535,9 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
           if (encaps_hdr.proto == IPPROTO_SCTP && !USI->ptech.rawsctpscan)
             continue;
         }
+        // If it didn't come from us, we don't care.
+        if (sockaddr_storage_cmp(USI->SourceSockAddr(), &encaps_hdr.src) != 0)
+          continue;
 
         hss = USI->findHost(&encaps_hdr.dst);
         if (!hss)
@@ -543,20 +546,12 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         probeI = hss->probes_outstanding.end();
         listsz = hss->num_probes_outstanding();
 
-        ss_len = sizeof(target_src);
-        hss->target->SourceSockAddr(&target_src, &ss_len);
-        ss_len = sizeof(target_dst);
-        hss->target->TargetSockAddr(&target_dst, &ss_len);
-
         /* Find the probe that provoked this response. */
         for (probenum = 0; probenum < listsz; probenum++) {
           probeI--;
           probe = *probeI;
 
-          if (probe->protocol() != encaps_hdr.proto ||
-              sockaddr_storage_cmp(&target_src, &hdr.dst) != 0 ||
-              sockaddr_storage_cmp(&target_src, &encaps_hdr.src) != 0 ||
-              sockaddr_storage_cmp(&target_dst, &encaps_hdr.dst) != 0)
+          if (probe->protocol() != encaps_hdr.proto)
             continue;
 
           if ((encaps_hdr.proto == IPPROTO_ICMP || encaps_hdr.proto == IPPROTO_ICMPV6)
@@ -601,7 +596,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
           // If it's Port or Proto unreachable and the address matches, it's up.
           if (((hdr.proto == IPPROTO_ICMP && (ping->code == 2 || ping->code == 3))
                 || (hdr.proto == IPPROTO_ICMPV6 && ping->code == 4))
-                && sockaddr_storage_cmp(&target_dst, &hdr.src) == 0) {
+                && sockaddr_storage_cmp(hss->target->TargetSockAddr(), &hdr.src) == 0) {
             /* The ICMP or ICMPv6 error came directly from the target, so it's up. */
             goodone = true;
             newstate = HOST_UP;
@@ -671,6 +666,8 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       setTargetMACIfAvailable(hss->target, &linkhdr, &hdr.src, 0);
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
+      u16 sport = ntohs(tcp->th_sport);
+      u16 dport = ntohs(tcp->th_dport);
 
       goodone = false;
 
@@ -679,7 +676,9 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         probeI--;
         probe = *probeI;
 
-        if (!tcp_probe_match(USI, probe, hss, tcp, &hdr.src, &hdr.dst, hdr.ipid))
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
+            continue;
+        if (!tcp_probe_match(USI, probe, tcp, &hdr.src, &hdr.dst, hdr.ipid))
           continue;
 
         goodone = true;
@@ -698,7 +697,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         }
 
         if (o.debugging)
-          log_write(LOG_STDOUT, "We got a TCP ping packet back from %s port %hu (trynum = %d)\n", inet_ntop_ez(&hdr.src, sizeof(hdr.src)), ntohs(tcp->th_sport), probe->get_tryno());
+          log_write(LOG_STDOUT, "We got a TCP ping packet back from %s port %hu (trynum = %d)\n", inet_ntop_ez(&hdr.src, sizeof(hdr.src)), sport, probe->get_tryno());
       }
     } else if (hdr.proto == IPPROTO_UDP && USI->ptech.rawudpscan) {
       const struct udp_hdr *udp = (struct udp_hdr *) data;
@@ -706,24 +705,19 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       hss = USI->findHost(&hdr.src);
       if (!hss)
         continue; // Not from a host that interests us
+
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
       goodone = false;
 
-      ss_len = sizeof(target_src);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
+      u16 sport = ntohs(udp->uh_sport);
+      u16 dport = ntohs(udp->uh_dport);
 
       for (probenum = 0; probenum < listsz && !goodone; probenum++) {
         probeI--;
         probe = *probeI;
 
-        if (o.af() != AF_INET || probe->protocol() != IPPROTO_UDP)
-          continue;
-
-        /* Ensure the connection info matches. */
-        if (probe->dport() != ntohs(udp->uh_sport) ||
-            probe->sport() != ntohs(udp->uh_dport) ||
-            sockaddr_storage_cmp(&target_src, &hdr.dst) != 0)
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
           continue;
 
         /* Sometimes we get false results when scanning localhost with
@@ -739,7 +733,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         current_reason = ER_UDPRESPONSE;
 
         if (o.debugging)
-          log_write(LOG_STDOUT, "In response to UDP-ping, we got UDP packet back from %s port %hu (trynum = %d)\n", inet_ntop_ez(&hdr.src, sizeof(hdr.src)), htons(udp->uh_sport), probe->get_tryno());
+          log_write(LOG_STDOUT, "In response to UDP-ping, we got UDP packet back from %s port %hu (trynum = %d)\n", inet_ntop_ez(&hdr.src, sizeof(hdr.src)), sport, probe->get_tryno());
       }
     } else if (hdr.proto == IPPROTO_SCTP && USI->ptech.rawsctpscan) {
       const struct sctp_hdr *sctp = (struct sctp_hdr *) data;
@@ -753,20 +747,14 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       listsz = hss->num_probes_outstanding();
       goodone = false;
 
-      ss_len = sizeof(target_dst);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
+      u16 sport = ntohs(sctp->sh_sport);
+      u16 dport = ntohs(sctp->sh_dport);
 
       for (probenum = 0; probenum < listsz && !goodone; probenum++) {
         probeI--;
         probe = *probeI;
 
-        if (o.af() != AF_INET || probe->protocol() != IPPROTO_SCTP)
-          continue;
-
-        /* Ensure the connection info matches. */
-        if (probe->dport() != ntohs(sctp->sh_sport) ||
-            probe->sport() != ntohs(sctp->sh_dport) ||
-            sockaddr_storage_cmp(&target_src, &hdr.dst) != 0)
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
           continue;
 
         /* Sometimes we get false results when scanning localhost with
@@ -822,12 +810,6 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
   } while (!goodone && !timedout);
 
   if (goodone && newstate != HOST_UNKNOWN) {
-    struct sockaddr_storage target_dst;
-    size_t ss_len;
-
-    ss_len = sizeof(target_dst);
-    hss->target->TargetSockAddr(&target_dst, &ss_len);
-
     if (probe->isPing())
       ultrascan_ping_update(USI, hss, probeI, &USI->now, adjust_timing);
     else {
@@ -839,7 +821,7 @@ int get_ping_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         setTargetMACIfAvailable(hss->target, &linkhdr, &hdr.src, 0);
       hss->target->reason.reason_id = current_reason;
       hss->target->reason.ttl = hdr.ttl;
-      if (sockaddr_storage_cmp(&hdr.src, &target_dst) != 0) {
+      if (sockaddr_storage_cmp(&hdr.src, hss->target->TargetSockAddr()) != 0) {
         hss->target->reason.set_ip_addr(&hdr.src);
       }
     }
@@ -869,7 +851,12 @@ void begin_sniffer(UltraScanInfo *USI, std::vector<Target *> &Targets) {
   if ((USI->pd = my_pcap_open_live(Targets[0]->deviceName(), 256,  (o.spoofsource) ? 1 : 0, pcap_selectable_fd_valid() ? 200 : 2)) == NULL)
     fatal("%s", PCAP_OPEN_ERRMSG);
 
+  int datalink = pcap_datalink(USI->pd);
+  if (datalink < 0) {
+    error("Warning: unable to determine data link for interface %s", Targets[0]->deviceName());
+  }
   if (USI->ping_scan_arp) {
+    assert(datalink == DLT_EN10MB);
     /* Some OSs including Windows 7 and Solaris 10 have been seen to send their
        ARP replies to the broadcast address, not to the (unicast) address that
        the request came from, therefore listening for ARP packets directed to
@@ -901,73 +888,26 @@ void begin_sniffer(UltraScanInfo *USI, std::vector<Target *> &Targets) {
              IPPROTO_ICMPV6, ICMPV6_NEIGHBOR_ADVERTISEMENT);
     pcap_filter.append(filterstr);
   } else if (USI->prot_scan || (USI->ping_scan && USI->ptech.rawprotoscan)) {
-    struct sockaddr_storage source;
-    size_t source_len;
-
-    source_len = sizeof(source);
-    Targets[0]->SourceSockAddr(&source, &source_len);
-
     pcap_filter = "dst host ";
-    pcap_filter += inet_ntop_ez(&source, sizeof(source));
+    pcap_filter += inet_ntop_ez(USI->SourceSockAddr(), sizeof(struct sockaddr_storage));
     if (doIndividual) {
       pcap_filter += " and (icmp or icmp6 or (";
       pcap_filter += dst_hosts;
       pcap_filter += "))";
     }
   } else if (USI->tcp_scan || USI->udp_scan || USI->sctp_scan || USI->ping_scan) {
-    struct sockaddr_storage source;
-    size_t source_len;
-    bool first = false;
-
-    source_len = sizeof(source);
-    Targets[0]->SourceSockAddr(&source, &source_len);
-
     pcap_filter = "dst host ";
-    pcap_filter += inet_ntop_ez(&source, sizeof(source));
-    pcap_filter += " and (icmp or icmp6";
+    pcap_filter += inet_ntop_ez(USI->SourceSockAddr(), sizeof(struct sockaddr_storage));
+    pcap_filter += " and (icmp or icmp6 or ";
     if (doIndividual) {
-      pcap_filter += " or (";
-      first = true;
-    }
-    if (USI->tcp_scan || (USI->ping_scan && USI->ptech.rawtcpscan)) {
-      if (!first) {
-        pcap_filter += " or ";
-      }
-      else if (doIndividual) {
-        pcap_filter += "(";
-      }
-      pcap_filter += "tcp";
-      first = false;
-    }
-    if (USI->udp_scan || (USI->ping_scan && USI->ptech.rawudpscan)) {
-      if (!first) {
-        pcap_filter += " or ";
-      }
-      else if (doIndividual) {
-        pcap_filter += "(";
-      }
-      pcap_filter += "udp";
-      first = false;
-    }
-    if (USI->sctp_scan || (USI->ping_scan && USI->ptech.rawsctpscan)) {
-      if (!first) {
-        pcap_filter += " or ";
-      }
-      else if (doIndividual) {
-        pcap_filter += "(";
-      }
-      pcap_filter += "sctp";
-      first = false;
+      pcap_filter += "((";
     }
+    // have to accept all of these because pingprobe could be any, regardless of scan type.
+    pcap_filter += "tcp or udp or sctp";
     if (doIndividual) {
-      if (!first) {
-        pcap_filter += ") and (";
-      }
+      pcap_filter += ") and (";
       pcap_filter += dst_hosts;
-      if (!first) {
-        pcap_filter += ")";
-      }
-      pcap_filter += ")";
+      pcap_filter += "))";
     }
     pcap_filter += ")";
   } else {
@@ -1000,7 +940,7 @@ UltraProbe *sendArpScanProbe(UltraScanInfo *USI, HostScanStats *hss,
   gettimeofday(&USI->now, NULL);
   probe->sent = USI->now;
   hss->probeSent(sizeof(frame));
-  if ((rc = eth_send(USI->ethsd, frame, sizeof(frame))) != sizeof(frame)) {
+  if ((rc = netutil_eth_send(USI->ethsd, frame, sizeof(frame))) != sizeof(frame)) {
     int err = socket_errno();
     error("WARNING: eth_send of ARP packet returned %i rather than expected %d (errno=%i: %s)", rc, (int) sizeof(frame), err, strerror(err));
   }
@@ -1029,13 +969,15 @@ UltraProbe *sendNDScanProbe(UltraScanInfo *USI, HostScanStats *hss,
   ns_dst_ip6 = *hss->target->v6hostip();
 
   if (USI->ethsd) {
-    unsigned char ns_dst_mac[6] = {0x33, 0x33, 0xff};
-    ns_dst_mac[3] = ns_dst_ip6.s6_addr[13];
-    ns_dst_mac[4] = ns_dst_ip6.s6_addr[14];
-    ns_dst_mac[5] = ns_dst_ip6.s6_addr[15];
+    if (netutil_eth_datalink(USI->ethsd) == DLT_EN10MB) {
+      unsigned char ns_dst_mac[6] = {0x33, 0x33, 0xff};
+      ns_dst_mac[3] = ns_dst_ip6.s6_addr[13];
+      ns_dst_mac[4] = ns_dst_ip6.s6_addr[14];
+      ns_dst_mac[5] = ns_dst_ip6.s6_addr[15];
 
-    memcpy(eth.srcmac, hss->target->SrcMACAddress(), 6);
-    memcpy(eth.dstmac, ns_dst_mac, 6);
+      memcpy(eth.srcmac, hss->target->SrcMACAddress(), 6);
+      memcpy(eth.dstmac, ns_dst_mac, 6);
+    }
     eth.ethsd = USI->ethsd;
     eth.devname[0] = '\0';
     ethptr = &eth;
@@ -1048,13 +990,7 @@ UltraProbe *sendNDScanProbe(UltraScanInfo *USI, HostScanStats *hss,
   multicast_prefix[12] = 0xff;
   memcpy(&ns_dst_ip6, multicast_prefix, sizeof(multicast_prefix));
 
-  struct sockaddr_storage source;
-  struct sockaddr_in6 *sin6;
-  size_t source_len;
-
-  source_len = sizeof(source);
-  hss->target->SourceSockAddr(&source, &source_len);
-  sin6 = (struct sockaddr_in6 *) &source;
+  const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) USI->SourceSockAddr();
 
   struct icmpv6_msg_nd ns_msg;
   ns_msg.icmpv6_flags = htons(0);
@@ -1225,8 +1161,10 @@ UltraProbe *sendIPScanProbe(UltraScanInfo *USI, HostScanStats *hss,
   u16 icmp_ident = (get_random_u16() % 0xffff) + 1;
 
   if (USI->ethsd) {
-    memcpy(eth.srcmac, hss->target->SrcMACAddress(), 6);
-    memcpy(eth.dstmac, hss->target->NextHopMACAddress(), 6);
+    if (netutil_eth_datalink(USI->ethsd) == DLT_EN10MB) {
+      memcpy(eth.srcmac, hss->target->SrcMACAddress(), 6);
+      memcpy(eth.dstmac, hss->target->NextHopMACAddress(), 6);
+    }
     eth.ethsd = USI->ethsd;
     eth.devname[0] = '\0';
     ethptr = &eth;
@@ -1292,8 +1230,8 @@ UltraProbe *sendIPScanProbe(UltraScanInfo *USI, HostScanStats *hss,
       }
     }
   } else if (pspec->type == PS_UDP) {
-    const char *payload;
-    size_t payload_length;
+    const u8 *payload;
+    int payload_length;
     u8 numpayloads = udp_payload_count(pspec->pd.udp.dport);
     // Even if no payloads, we can send with null payload
     numpayloads = MAX(numpayloads, 1);
@@ -1500,7 +1438,7 @@ bool get_arp_result(UltraScanInfo *USI, struct timeval *stime) {
     if (rc == -1)
       fatal("Received -1 response from read_arp_reply_pcap");
     if (rc == 0) {
-      if (TIMEVAL_SUBTRACT(*stime, USI->now) < 0) {
+      if (TIMEVAL_BEFORE(*stime, USI->now)) {
         timedout = true;
         break;
       } else {
@@ -1579,7 +1517,7 @@ bool get_ns_result(UltraScanInfo *USI, struct timeval *stime) {
     if (rc == -1)
       fatal("Received -1 response from read_arp_reply_pcap");
     if (rc == 0) {
-      if (TIMEVAL_SUBTRACT(*stime, USI->now) < 0) {
+      if (TIMEVAL_BEFORE(*stime, USI->now)) {
         timedout = true;
         break;
       } else {
@@ -1645,6 +1583,7 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
   bool goodone = false;
   bool timedout = false;
   bool adjust_timing = true;
+  bool from_target = true;
   struct timeval rcvdtime;
   struct link_header linkhdr;
   unsigned int bytes;
@@ -1676,7 +1615,7 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       to_usec = 2000;
     ip_tmp = (struct ip *) readip_pcap(USI->pd, &bytes, to_usec, &rcvdtime, &linkhdr, true);
     gettimeofday(&USI->now, NULL);
-    if (!ip_tmp && TIMEVAL_SUBTRACT(*stime, USI->now) < 0) {
+    if (!ip_tmp && TIMEVAL_BEFORE(*stime, USI->now)) {
       timedout = true;
       break;
     } else if (!ip_tmp)
@@ -1688,13 +1627,13 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       timedout = true;
     }
 
-    struct sockaddr_storage target_src, target_dst;
-    size_t ss_len;
-
     datalen = bytes;
     data = ip_get_data(ip_tmp, &datalen, &hdr);
     if (data == NULL)
       continue;
+    /* Ensure the connection info matches. */
+    if (sockaddr_storage_cmp(USI->SourceSockAddr(), &hdr.dst) != 0)
+      continue;
 
     if (USI->prot_scan) {
       hss = USI->findHost(&hdr.src);
@@ -1737,6 +1676,8 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       setTargetMACIfAvailable(hss->target, &linkhdr, &hdr.src, 0);
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
+      u16 sport = ntohs(tcp->th_sport);
+      u16 dport = ntohs(tcp->th_dport);
 
       goodone = false;
 
@@ -1745,7 +1686,9 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         probeI--;
         probe = *probeI;
 
-        if (!tcp_probe_match(USI, probe, hss, tcp, &hdr.src, &hdr.dst, hdr.ipid))
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
+            continue;
+        if (!tcp_probe_match(USI, probe, tcp, &hdr.src, &hdr.dst, hdr.ipid))
           continue;
 
         if (!probe->isPing()) {
@@ -1788,20 +1731,15 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
 
       goodone = false;
 
-      ss_len = sizeof(target_src);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
+      u16 sport = ntohs(sctp->sh_sport);
+      u16 dport = ntohs(sctp->sh_dport);
 
       /* Find the probe that provoked this response. */
       for (probenum = 0; probenum < listsz && !goodone; probenum++) {
         probeI--;
         probe = *probeI;
 
-        if (probe->protocol() != IPPROTO_SCTP)
-          continue;
-        /* Ensure the connection info matches. */
-        if (probe->dport() != ntohs(sctp->sh_sport)
-            || probe->sport() != ntohs(sctp->sh_dport)
-            || sockaddr_storage_cmp(&target_src, &hdr.dst) != 0)
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
           continue;
 
         /* Sometimes we get false results when scanning localhost with
@@ -1878,6 +1816,10 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       if (USI->sctp_scan && encaps_hdr.proto != IPPROTO_SCTP)
         continue;
 
+      // If it didn't come from us, we don't care.
+      if (sockaddr_storage_cmp(USI->SourceSockAddr(), &encaps_hdr.src) != 0)
+        continue;
+
       /* ensure this packet relates to a packet to the host
       we are scanning ... */
       hss = USI->findHost(&encaps_hdr.dst);
@@ -1885,20 +1827,14 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         continue; // Not from a host that interests us
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
-
-      ss_len = sizeof(target_src);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
-      ss_len = sizeof(target_dst);
-      hss->target->TargetSockAddr(&target_dst, &ss_len);
+      from_target = sockaddr_storage_cmp(hss->target->TargetSockAddr(), &hdr.src) == 0;
 
       goodone = false;
       /* Find the matching probe */
       for (probenum = 0; probenum < listsz && !goodone; probenum++) {
         probeI--;
         probe = *probeI;
-        if (probe->protocol() != encaps_hdr.proto ||
-            sockaddr_storage_cmp(&target_src, &encaps_hdr.src) != 0 ||
-            sockaddr_storage_cmp(&target_dst, &encaps_hdr.dst) != 0)
+        if (probe->protocol() != encaps_hdr.proto)
           continue;
 
         if (encaps_hdr.proto == IPPROTO_TCP && !USI->prot_scan) {
@@ -1932,18 +1868,15 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
             newstate = PORT_FILTERED;
             break;
           case 2: /* protocol unreachable */
-            if (USI->scantype == IPPROT_SCAN &&
-                sockaddr_storage_cmp(&target_dst, &hdr.src) == 0) {
+            if (USI->scantype == IPPROT_SCAN && from_target)
               newstate = PORT_CLOSED;
-            } else
+            else
               newstate = PORT_FILTERED;
             break;
           case 3: /* Port unreach */
-            if (USI->scantype == UDP_SCAN &&
-                sockaddr_storage_cmp(&target_dst, &hdr.src) == 0)
+            if (from_target && USI->scantype == UDP_SCAN)
               newstate = PORT_CLOSED;
-            else if (USI->scantype == IPPROT_SCAN &&
-                     sockaddr_storage_cmp(&target_dst, &hdr.src) == 0)
+            else if (from_target && USI->scantype == IPPROT_SCAN)
               newstate = PORT_OPEN;
             else
               newstate = PORT_FILTERED;
@@ -2007,6 +1940,10 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
       if (USI->sctp_scan && encaps_hdr.proto != IPPROTO_SCTP)
         continue;
 
+      // If it didn't come from us, we don't care.
+      if (sockaddr_storage_cmp(USI->SourceSockAddr(), &encaps_hdr.src) != 0)
+        continue;
+
       /* ensure this packet relates to a packet to the host
       we are scanning ... */
       hss = USI->findHost(&encaps_hdr.dst);
@@ -2014,20 +1951,14 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         continue; // Not from a host that interests us
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
-
-      ss_len = sizeof(target_src);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
-      ss_len = sizeof(target_dst);
-      hss->target->TargetSockAddr(&target_dst, &ss_len);
+      from_target = sockaddr_storage_cmp(hss->target->TargetSockAddr(), &hdr.src) == 0;
 
       goodone = false;
       /* Find the matching probe */
       for (probenum = 0; probenum < listsz && !goodone; probenum++) {
         probeI--;
         probe = *probeI;
-        if (probe->protocol() != encaps_hdr.proto ||
-            sockaddr_storage_cmp(&target_src, &encaps_hdr.src) != 0 ||
-            sockaddr_storage_cmp(&target_dst, &encaps_hdr.dst) != 0)
+        if (probe->protocol() != encaps_hdr.proto)
           continue;
 
         if (encaps_hdr.proto == IPPROTO_TCP && !USI->prot_scan) {
@@ -2080,11 +2011,9 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
             break;
           case ICMPV6_UNREACH_PORT:
             current_reason = ER_PORTUNREACH;
-            if (USI->scantype == UDP_SCAN &&
-                sockaddr_storage_cmp(&target_dst, &hdr.src) == 0)
+            if (from_target && USI->scantype == UDP_SCAN)
               newstate = PORT_CLOSED;
-            else if (USI->scantype == IPPROT_SCAN &&
-                     sockaddr_storage_cmp(&target_dst, &hdr.src) == 0)
+            else if (from_target && USI->scantype == IPPROT_SCAN)
               newstate = PORT_OPEN;
             else
               newstate = PORT_FILTERED;
@@ -2103,12 +2032,10 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
             newstate = PORT_OPEN;
             break;
           case ICMPV6_PARAMPROBLEM_NEXTHEADER:
-            if (USI->scantype == IPPROT_SCAN &&
-                sockaddr_storage_cmp(&target_dst, &hdr.src) == 0) {
+            if (from_target && USI->scantype == IPPROT_SCAN)
               newstate = PORT_CLOSED;
-            } else {
+            else
               newstate = PORT_FILTERED;
-            }
             break;
           default:
             error("Unexpected ICMPv6 type/code %d/%d unreachable packet:\n",
@@ -2136,8 +2063,8 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         continue; // Not from a host that interests us
       probeI = hss->probes_outstanding.end();
       listsz = hss->num_probes_outstanding();
-      ss_len = sizeof(target_src);
-      hss->target->SourceSockAddr(&target_src, &ss_len);
+      u16 sport = ntohs(udp->uh_sport);
+      u16 dport = ntohs(udp->uh_dport);
 
       goodone = false;
 
@@ -2146,13 +2073,7 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         probe = *probeI;
         newstate = PORT_UNKNOWN;
 
-        if (probe->protocol() != IPPROTO_UDP)
-          continue;
-
-        /* Ensure the connection info matches. */
-        if (probe->dport() != ntohs(udp->uh_sport) ||
-            probe->sport() != ntohs(udp->uh_dport) ||
-            sockaddr_storage_cmp(&target_src, &hdr.dst) != 0)
+        if (!probe->check_proto_port(hdr.proto, dport, sport))
           continue;
 
         /* Sometimes we get false results when scanning localhost with
@@ -2166,18 +2087,20 @@ bool get_pcap_result(UltraScanInfo *USI, struct timeval *stime) {
         newstate = PORT_OPEN;
         current_reason = ER_UDPRESPONSE;
         goodone = true;
+        /* Store the data response in case service_scan wants it later */
+        if (datalen > UDP_HDR_LEN) {
+          struct EarlySvcResponse *esr = (EarlySvcResponse *) safe_zalloc(sizeof(struct EarlySvcResponse) + datalen - UDP_HDR_LEN);
+          esr->pspec = *(probe->pspec());
+          esr->len = datalen - UDP_HDR_LEN;
+          memcpy(esr->data, (u8 *)data + UDP_HDR_LEN, esr->len);
+          hss->target->earlySvcResponses.push_back(esr);
+        }
       }
     } else continue; /* Unexpected protocol */
   } while (!goodone && !timedout);
 
   if (goodone) {
-    struct sockaddr_storage target_dst;
-    size_t ss_len;
-
-    ss_len = sizeof(target_dst);
-    hss->target->TargetSockAddr(&target_dst, &ss_len);
-
-    if (sockaddr_storage_cmp(&hdr.src, &target_dst) == 0)
+    if (from_target)
       reason_sip.ss_family = AF_UNSPEC;
     else
       reason_sip = hdr.src;
