@@ -2,7 +2,7 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *
-# * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+# * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
 # * Project"). Nmap is also a registered trademark of the Nmap Project.
 # *
 # * This program is distributed under the terms of the Nmap Public Source
@@ -129,8 +129,8 @@ class About(HIGDialog):
     have roughly the same feel as Gtk.AboutDialog."""
     def __init__(self):
         HIGDialog.__init__(self)
-        self.set_title(_("About %s and %s") % (
-            NMAP_DISPLAY_NAME, APP_DISPLAY_NAME))
+        self.set_title(_("About %(nmap)s and %(zenmap)s") % {
+            'nmap': NMAP_DISPLAY_NAME, 'zenmap': APP_DISPLAY_NAME })
 
         self.vbox.set_border_width(12)
         self.vbox.set_spacing(12)
@@ -153,14 +153,16 @@ class About(HIGDialog):
         self.vbox.pack_start(entry, True, True, 0)
 
         entry = _program_entry(APP_DISPLAY_NAME, APP_WEB_SITE, _(
-            "%s is a multi-platform graphical %s frontend and results viewer. "
-            "It was originally derived from %s.") % (
-                APP_DISPLAY_NAME, NMAP_DISPLAY_NAME, UMIT_DISPLAY_NAME))
+            "%(zenmap)s is a multi-platform graphical %(nmap)s frontend and results viewer. "
+            "It was originally derived from %(umit)s.") % {
+                'zenmap': APP_DISPLAY_NAME,
+                'nmap': NMAP_DISPLAY_NAME,
+                'umit': UMIT_DISPLAY_NAME})
         self.vbox.pack_start(entry, True, True, 0)
 
         entry = _program_entry(UMIT_DISPLAY_NAME, UMIT_WEB_SITE, _(
-            "%s is an %s GUI created as part of the Nmap/Google Summer "
-            "of Code program.") % (UMIT_DISPLAY_NAME, NMAP_DISPLAY_NAME))
+            "%(umit)s is a %(nmap)s GUI created as part of the Nmap/Google Summer "
+            "of Code program.") % {'umit': UMIT_DISPLAY_NAME, 'nmap': NMAP_DISPLAY_NAME})
         button = Gtk.Button.new_with_label(_("%s credits") % UMIT_DISPLAY_NAME)
         button.connect("clicked", self._show_umit_credits)
         entry.hbox.pack_start(button, False, True, 0)
