@@ -2,7 +2,7 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *
-# * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+# * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
 # * Project"). Nmap is also a registered trademark of the Nmap Project.
 # *
 # * This program is distributed under the terms of the Nmap Public Source
@@ -205,9 +205,9 @@ class ScanChooser(HIGVBox):
                 alert = HIGAlertDialog(
                     message_format='<b>%s</b>' % _(
                         'Cannot open selected file'),
-                    secondary_text=_("""\
-                        This error occurred while trying to open the file:
-                        %s""") % str(e))
+                    secondary_text=_(
+                        "This error occurred while trying to open the file:\n%s"
+                        ) % str(e))
                 alert.run()
                 alert.destroy()
                 return False
@@ -377,9 +377,10 @@ class DiffWindow(Gtk.Window):
                 error_text = _(
                     "The ndiff process terminated with status code %d."
                     ) % status
-                stderr = self.ndiff_process.stderr.read()
-                if len(stderr) > 0:
-                    error_text += "\n\n" + stderr
+                if self.ndiff_process.stderr:
+                    stderr = self.ndiff_process.stderr.read()
+                    if len(stderr) > 0:
+                        error_text += "\n\n" + stderr
                 alert = HIGAlertDialog(
                     message_format=_("Error running ndiff"),
                     secondary_text=error_text)
